@@ -43,39 +43,35 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
 
     uint32_t tmp_val;
     uint32_t midi_type;
-    uint32_t track_size;
     uint8_t **tracks;
+    uint32_t *track_size;
     uint32_t end_of_tracks = 0;
     uint32_t no_tracks;
     uint32_t i;
     uint32_t divisions = 96;
     uint32_t tempo = 500000;
-    float samples_per_delta_f = 0.0;
+    float samples_per_delta_f = 0;
 
     uint32_t sample_count = 0;
-    float sample_count_f = 0.0;
-    float sample_remainder = 0.0;
+    float sample_count_f = 0;
+    float sample_remainder = 0;
     uint8_t *sysex_store = NULL;
-//  uint32_t sysex_store_len = 0;
 
     uint32_t *track_delta;
     uint8_t *track_end;
     uint32_t smallest_delta = 0;
     uint32_t subtract_delta = 0;
-//  uint32_t tmp_length = 0;
-//  uint8_t current_event = 0;
-//  uint8_t current_event_ch = 0;
     uint8_t *running_event;
     uint32_t setup_ret = 0;
 
     if (midi_size < 14) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
         return (NULL);
     }
 
     if (!memcmp(midi_data, "RIFF", 4)) {
         if (midi_size < 34) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
             return (NULL);
         }
         midi_data += 20;
@@ -83,7 +79,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     }
 
     if (memcmp(midi_data, "MThd", 4)) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_NOT_MIDI, NULL, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_MIDI, NULL, 0);
         return (NULL);
     }
     midi_data += 4;
@@ -98,7 +94,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     tmp_val |= *midi_data++;
     midi_size -= 4;
     if (tmp_val != 6) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, NULL, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, NULL, 0);
         return (NULL);
     }
 
@@ -109,7 +105,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     tmp_val |= *midi_data++;
     midi_size -= 2;
     if (tmp_val > 2) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_INVALID, NULL, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, NULL, 0);
         return (NULL);
     }
     midi_type = tmp_val;
@@ -121,7 +117,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     tmp_val |= *midi_data++;
     midi_size -= 2;
     if (tmp_val < 1) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(no tracks)", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(no tracks)", 0);
         return (NULL);
     }
     no_tracks = tmp_val;
@@ -130,7 +126,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
      * Check that type 0 midi file has only 1 track
      */
     if ((midi_type == 0) && (no_tracks > 1)) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_INVALID, "(expected 1 track for type 0 midi file, found more)", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, "(expected 1 track for type 0 midi file, found more)", 0);
         return (NULL);
     }
 
@@ -141,7 +137,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     divisions |= *midi_data++;
     midi_size -= 2;
     if (divisions & 0x00008000) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_INVALID, NULL, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, NULL, 0);
         return (NULL);
     }
 
@@ -150,46 +146,49 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     mdi = _WM_initMDI();
     _WM_midi_setup_divisions(mdi,divisions);
 
-    tracks = malloc(sizeof(uint8_t *) * no_tracks);
-    track_delta = malloc(sizeof(uint32_t) * no_tracks);
-    track_end = malloc(sizeof(uint8_t) * no_tracks);
-    running_event = malloc(sizeof(uint8_t) * no_tracks);
+    tracks = (uint8_t **) malloc(sizeof(uint8_t *) * no_tracks);
+    track_size = (uint32_t *) malloc(sizeof(uint32_t) * no_tracks);
+    track_delta = (uint32_t *) malloc(sizeof(uint32_t) * no_tracks);
+    track_end = (uint8_t *) malloc(sizeof(uint8_t) * no_tracks);
+    running_event = (uint8_t *) malloc(sizeof(uint8_t) * no_tracks);
 
     smallest_delta = 0xffffffff;
     for (i = 0; i < no_tracks; i++) {
         if (midi_size < 8) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
             goto _end;
         }
         if (memcmp(midi_data, "MTrk", 4) != 0) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(missing track header)", 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(missing track header)", 0);
             goto _end;
         }
         midi_data += 4;
         midi_size -= 4;
 
-        track_size = *midi_data++ << 24;
-        track_size |= *midi_data++ << 16;
-        track_size |= *midi_data++ << 8;
-        track_size |= *midi_data++;
+        /* track size */
+        tmp_val = *midi_data++ << 24;
+        tmp_val |= *midi_data++ << 16;
+        tmp_val |= *midi_data++ << 8;
+        tmp_val |= *midi_data++;
         midi_size -= 4;
-        if (midi_size < track_size) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+        if (midi_size < tmp_val) {
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
             goto _end;
         }
-        if (track_size < 3) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(bad track size)", 0);
+        if (tmp_val < 3) {
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(bad track size)", 0);
             goto _end;
         }
-        if ((midi_data[track_size - 3] != 0xFF)
-                || (midi_data[track_size - 2] != 0x2F)
-                || (midi_data[track_size - 1] != 0x00)) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(missing EOT)", 0);
+        if ((midi_data[tmp_val - 3] != 0xFF)
+                || (midi_data[tmp_val - 2] != 0x2F)
+                || (midi_data[tmp_val - 1] != 0x00)) {
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(missing EOT)", 0);
             goto _end;
         }
         tracks[i] = midi_data;
-        midi_data += track_size;
-        midi_size -= track_size;
+        track_size[i] = tmp_val;
+        midi_data += tmp_val;
+        midi_size -= tmp_val;
         track_end[i] = 0;
         running_event[i] = 0;
         track_delta[i] = 0;
@@ -197,9 +196,11 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
         while (*tracks[i] > 0x7F) {
             track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
             tracks[i]++;
+            track_size[i]--;
         }
         track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
         tracks[i]++;
+        track_size[i]--;
 
         if (midi_type == 1 ) {
             if (track_delta[i] < smallest_delta) {
@@ -243,9 +244,8 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                     }
                 }
                 do {
-                    setup_ret = _WM_SetupMidiEvent(mdi, tracks[i], running_event[i]);
+                    setup_ret = _WM_SetupMidiEvent(mdi, tracks[i], track_size[i], running_event[i]);
                     if (setup_ret == 0) {
-                        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(missing event)", 0);
                         goto _end;
                     }
                     if (tracks[i][0] > 0x7f) {
@@ -260,6 +260,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                             end_of_tracks++;
                             track_end[i] = 1;
                             tracks[i] += 3;
+                            track_size[i] -= 3;
                             goto NEXT_TRACK;
                         } else if ((tracks[i][0] == 0xff) && (tracks[i][1] == 0x51) && (tracks[i][2] == 0x03)) {
                             /* Tempo */
@@ -271,15 +272,23 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                         }
                     }
                     tracks[i] += setup_ret;
+                    track_size[i] -= setup_ret;
 
                     if (*tracks[i] > 0x7f) {
                         do {
+                            if (!track_size[i]) break;
                             track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
                             tracks[i]++;
+                            track_size[i]--;
                         } while (*tracks[i] > 0x7f);
                     }
+                    if (!track_size[i]) {
+                        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+                        goto _end;
+                    }
                     track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
                     tracks[i]++;
+                    track_size[i]--;
                 } while (!track_delta[i]);
                 if ((!smallest_delta) || (smallest_delta > track_delta[i])) {
                     smallest_delta = track_delta[i];
@@ -301,13 +310,12 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
         if (midi_type == 2) {
             mdi->is_type2 = 1;
         }
-        sample_remainder = 0.0;
+        sample_remainder = 0;
         for (i = 0; i < no_tracks; i++) {
             running_event[i] = 0;
             do {
-                setup_ret = _WM_SetupMidiEvent(mdi, tracks[i], running_event[i]);
+                setup_ret = _WM_SetupMidiEvent(mdi, tracks[i], track_size[i], running_event[i]);
                 if (setup_ret == 0) {
-                    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(missing event)", 0);
                     goto _end;
                 }
                 if (tracks[i][0] > 0x7f) {
@@ -331,16 +339,24 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
                     }
                 }
                 tracks[i] += setup_ret;
+                track_size[i] -= setup_ret;
 
                 track_delta[i] = 0;
                 if (*tracks[i] > 0x7f) {
                     do {
+                        if (!track_size[i]) break;
                         track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
                         tracks[i]++;
+                        track_size[i]--;
                     } while (*tracks[i] > 0x7f);
                 }
+                if (!track_size[i]) {
+                    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+                    goto _end;
+                }
                 track_delta[i] = (track_delta[i] << 7) + (*tracks[i] & 0x7F);
                 tracks[i]++;
+                track_size[i]--;
 
                 sample_count_f = (((float) track_delta[i] * samples_per_delta_f)
                                   + sample_remainder);
@@ -358,7 +374,7 @@ _WM_ParseNewMidi(uint8_t *midi_data, uint32_t midi_size) {
     if ((mdi->reverb = _WM_init_reverb(_WM_SampleRate, _WM_reverb_room_width,
             _WM_reverb_room_length, _WM_reverb_listen_posx, _WM_reverb_listen_posy))
           == NULL) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to init reverb", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to init reverb", 0);
         goto _end;
     }
 
@@ -374,6 +390,7 @@ _end:   free(sysex_store);
     free(track_delta);
     free(running_event);
     free(tracks);
+    free(track_size);
     if (mdi->reverb) return (mdi);
     _WM_freeMDI(mdi);
     return (NULL);
@@ -402,16 +419,16 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
     uint8_t running_event = 0;
     uint32_t divisions = 96;
     uint32_t tempo = 500000;
-    float samples_per_tick = 0.0;
+    float samples_per_tick = 0;
     uint32_t value = 0;
-    float value_f = 0.0;
+    float value_f = 0;
     struct _event *event = mdi->events;
     uint32_t track_size = 0;
     uint32_t track_start = 0;
     uint32_t track_count = 0;
 
     if (!mdi->event_count) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CONVERT, "(No events to convert)", 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CONVERT, "(No events to convert)", 0);
         return -1;
     }
 
@@ -421,7 +438,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
      Note: This isn't accurate but will allow enough space for
             events plus delta values.
      */
-    (*out) = malloc (sizeof(uint8_t) * (mdi->event_count * 12));
+    (*out) = (uint8_t *) malloc (sizeof(uint8_t) * (mdi->event_count * 12));
 
     /* Midi Header */
     (*out)[0] = 'M';
@@ -679,7 +696,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
         } else if (event->do_event == _WM_do_sysex_roland_drum_track) {
             // DEBUG
             // fprintf(stderr,"Sysex Roland Drum Track: %u %.4x\r\n",event->event_data.channel, event->event_data.data);
-            int8_t foo[] = {0xf0, 0x09, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x15, 0x00, 0xf7};
+            uint8_t foo[] = {0xf0, 0x09, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x15, 0x00, 0xf7};
             uint8_t foo_ch = event->event_data.channel;
             if (foo_ch == 9) {
                 foo_ch = 0;
@@ -694,21 +711,21 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
         } else if (event->do_event == _WM_do_sysex_gm_reset) {
             // DEBUG
             // fprintf(stderr,"Sysex GM Reset\r\n");
-            int8_t foo[] = {0xf0, 0x05, 0x7e, 0x7f, 0x09, 0x01, 0xf7};
+            uint8_t foo[] = {0xf0, 0x05, 0x7e, 0x7f, 0x09, 0x01, 0xf7};
             memcpy(&((*out)[out_ofs]),foo,7);
             out_ofs += 7;
             running_event = 0;
         } else if (event->do_event == _WM_do_sysex_roland_reset) {
             // DEBUG
             // fprintf(stderr,"Sysex Roland Reset\r\n");
-            int8_t foo[] = {0xf0, 0x0a, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41, 0xf7};
+            uint8_t foo[] = {0xf0, 0x0a, 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41, 0xf7};
             memcpy(&((*out)[out_ofs]),foo,12);
             out_ofs += 12;
             running_event = 0;
         } else if (event->do_event == _WM_do_sysex_yamaha_reset) {
             // DEBUG
             // fprintf(stderr,"Sysex Yamaha Reset\r\n");
-            int8_t foo[] = {0xf0, 0x08, 0x43, 0x10, 0x4c, 0x00, 0x00, 0x7e, 0x00, 0xf7};
+            uint8_t foo[] = {0xf0, 0x08, 0x43, 0x10, 0x4c, 0x00, 0x00, 0x7e, 0x00, 0xf7};
             memcpy(&((*out)[out_ofs]),foo,10);
             out_ofs += 10;
             running_event = 0;
@@ -876,7 +893,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
         }
 
         value_f = (float)event->samples_to_next / samples_per_tick;
-        value = (uint32_t)(value_f + 0.5);
+        value = (uint32_t) (value_f + 0.5f);
 
         //DEBUG
         //fprintf(stderr,"\rDEBUG: STN %i, SPD %f, Delta %i\r\n", event->samples_to_next, samples_per_delta_f, value);
@@ -911,7 +928,7 @@ _WM_Event2Midi(struct _mdi *mdi, uint8_t **out, uint32_t *outsize) {
     (*out)[10] = (track_count >> 8) & 0xff;
     (*out)[11] = track_count & 0xff;
 
-    (*out) = realloc((*out), out_ofs);
+    (*out) = (uint8_t *) realloc((*out), out_ofs);
     (*outsize) = out_ofs;
 
     return 0;
