@@ -53,8 +53,7 @@
 #define GUSPAT_END_DEBUG()
 #endif
 
-/*
- * sample data conversion functions
+/* sample data conversion functions
  * convert data to signed shorts
  */
 
@@ -65,7 +64,7 @@ static int convert_8s(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((gus_sample->data_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((gus_sample->data_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -74,7 +73,7 @@ static int convert_8s(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, 0, "calloc failed", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, 0, "calloc failed", errno);
 
     return -1;
 }
@@ -92,7 +91,7 @@ static int convert_8sp(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((new_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((new_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -127,7 +126,7 @@ static int convert_8sp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -139,7 +138,7 @@ static int convert_8sr(uint8_t *data, struct _sample *gus_sample) {
     uint32_t tmp_loop = 0;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((gus_sample->data_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((gus_sample->data_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data + gus_sample->data_length - 1;
         do {
@@ -153,7 +152,7 @@ static int convert_8sr(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -170,7 +169,7 @@ static int convert_8srp(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((new_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((new_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -204,7 +203,7 @@ static int convert_8srp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -215,7 +214,7 @@ static int convert_8u(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((gus_sample->data_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((gus_sample->data_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -224,7 +223,7 @@ static int convert_8u(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -241,7 +240,7 @@ static int convert_8up(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((new_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((new_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -276,7 +275,7 @@ static int convert_8up(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -288,7 +287,7 @@ static int convert_8ur(uint8_t *data, struct _sample *gus_sample) {
     uint32_t tmp_loop = 0;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((gus_sample->data_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((gus_sample->data_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data + gus_sample->data_length - 1;
         do {
@@ -302,7 +301,7 @@ static int convert_8ur(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE | SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -319,7 +318,7 @@ static int convert_8urp(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc((new_length + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc((new_length + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -352,7 +351,7 @@ static int convert_8urp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -363,7 +362,7 @@ static int convert_16s(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -376,7 +375,7 @@ static int convert_16s(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->data_length >>= 1;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -393,7 +392,7 @@ static int convert_16sp(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((new_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((new_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -436,7 +435,7 @@ static int convert_16sp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -448,7 +447,7 @@ static int convert_16sr(uint8_t *data, struct _sample *gus_sample) {
     uint32_t tmp_loop = 0;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data + (gus_sample->data_length >> 1) - 1;
         do {
@@ -466,7 +465,7 @@ static int convert_16sr(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -483,7 +482,7 @@ static int convert_16srp(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((new_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((new_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -521,7 +520,7 @@ static int convert_16srp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -532,7 +531,7 @@ static int convert_16u(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -545,7 +544,7 @@ static int convert_16u(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -562,7 +561,7 @@ static int convert_16up(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((new_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((new_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -605,7 +604,7 @@ static int convert_16up(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -617,7 +616,7 @@ static int convert_16ur(uint8_t *data, struct _sample *gus_sample) {
     uint32_t tmp_loop = 0;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((gus_sample->data_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data + (gus_sample->data_length >> 1) - 1;
         do {
@@ -635,7 +634,7 @@ static int convert_16ur(uint8_t *data, struct _sample *gus_sample) {
         gus_sample->modes ^= SAMPLE_REVERSE | SAMPLE_UNSIGNED;
         return 0;
     }
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -652,7 +651,7 @@ static int convert_16urp(uint8_t *data, struct _sample *gus_sample) {
     int16_t *write_data_b = NULL;
 
     SAMPLE_CONVERT_DEBUG(__FUNCTION__);
-    gus_sample->data = calloc(((new_length >> 1) + 2), sizeof(int16_t));
+    gus_sample->data = (int16_t *) calloc(((new_length >> 1) + 2), sizeof(int16_t));
     if (__builtin_expect((gus_sample->data != NULL), 1)) {
         write_data = gus_sample->data;
         do {
@@ -690,7 +689,7 @@ static int convert_16urp(uint8_t *data, struct _sample *gus_sample) {
         return 0;
     }
 
-    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
+    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, "to parse sample", errno);
     return -1;
 }
 
@@ -733,30 +732,23 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
         return NULL;
     }
     if (gus_size < 239) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_LOAD, filename, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, filename, 0);
         free(gus_patch);
         return NULL;
     }
     if (memcmp(gus_patch, "GF1PATCH110\0ID#000002", 22)
             && memcmp(gus_patch, "GF1PATCH100\0ID#000002", 22)) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_INVALID, "(unsupported format)",
-                0);
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_LOAD, filename, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, filename, 0);
         free(gus_patch);
         return NULL;
     }
     if (gus_patch[82] > 1) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_INVALID, "(unsupported format)",
-                0);
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_LOAD, filename, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, filename, 0);
         free(gus_patch);
         return NULL;
     }
     if (gus_patch[151] > 1) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_INVALID, "(unsupported format)",
-                0);
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_LOAD, filename, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_INVALID, filename, 0);
         free(gus_patch);
         return NULL;
     }
@@ -769,15 +761,14 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
     while (no_of_samples) {
         uint32_t tmp_cnt;
         if (first_gus_sample == NULL) {
-            first_gus_sample = malloc(sizeof(struct _sample));
+            first_gus_sample = (struct _sample *) malloc(sizeof(struct _sample));
             gus_sample = first_gus_sample;
         } else {
-            gus_sample->next = malloc(sizeof(struct _sample));
+            gus_sample->next = (struct _sample *) malloc(sizeof(struct _sample));
             gus_sample = gus_sample->next;
         }
         if (gus_sample == NULL) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_MEM, NULL, 0);
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_LOAD, filename, 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_MEM, filename, 0);
             free(gus_patch);
             return NULL;
         }
@@ -834,107 +825,39 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
                             | ((gus_sample->loop_fraction & 0xf0) >> 4);
         }
 
-        /*
-         FIXME: Experimental Hacky Fix
-
-         This looks for "dodgy" release envelope settings that faulty editors
-         may have set and attempts to corrects it.
-
-         if (fix_release)
-         Lets make this automatic ...
-         */
-        {
-            /*
-             After studying faulty gus_pats this way may work better
-             Testing to determine if any further adjustments are required
-             */
-            if (env_time_table[gus_patch[gus_ptr + 40]] < env_time_table[gus_patch[gus_ptr + 41]]) {
-                uint8_t tmp_hack_rate = 0;
-
-                if (env_time_table[gus_patch[gus_ptr + 41]] < env_time_table[gus_patch[gus_ptr + 42]]) {
-                    // 1 2 3
-                    tmp_hack_rate = gus_patch[gus_ptr + 40];
-                    gus_patch[gus_ptr + 40] = gus_patch[gus_ptr + 42];
-                    gus_patch[gus_ptr + 42] = tmp_hack_rate;
-                } else if (env_time_table[gus_patch[gus_ptr + 41]] == env_time_table[gus_patch[gus_ptr + 42]]) {
-                    // 1 2 2
-                    tmp_hack_rate = gus_patch[gus_ptr + 40];
-                    gus_patch[gus_ptr + 40] = gus_patch[gus_ptr + 42];
-                    gus_patch[gus_ptr + 41] = gus_patch[gus_ptr + 42];
-                    gus_patch[gus_ptr + 42] = tmp_hack_rate;
-
-                } else {
-                    if (env_time_table[gus_patch[gus_ptr + 40]] < env_time_table[gus_patch[gus_ptr + 42]]) {
-                        // 1 3 2
-                        tmp_hack_rate = gus_patch[gus_ptr + 40];
-                        gus_patch[gus_ptr + 40] = gus_patch[gus_ptr + 41];
-                        gus_patch[gus_ptr + 41] = gus_patch[gus_ptr + 42];
-                        gus_patch[gus_ptr + 42] = tmp_hack_rate;
-                    } else {
-                        // 2 3 1 or 1 2 1
-                        tmp_hack_rate = gus_patch[gus_ptr + 40];
-                        gus_patch[gus_ptr + 40] = gus_patch[gus_ptr + 41];
-                        gus_patch[gus_ptr + 41] = tmp_hack_rate;
-                    }
-                }
-            } else if (env_time_table[gus_patch[gus_ptr + 41]] < env_time_table[gus_patch[gus_ptr + 42]]) {
-                uint8_t tmp_hack_rate = 0;
-
-                if (env_time_table[gus_patch[gus_ptr + 40]] < env_time_table[gus_patch[gus_ptr + 42]]) {
-                    // 2 1 3
-                    tmp_hack_rate = gus_patch[gus_ptr + 40];
-                    gus_patch[gus_ptr + 40] = gus_patch[gus_ptr + 42];
-                    gus_patch[gus_ptr + 42] = gus_patch[gus_ptr + 41];
-                    gus_patch[gus_ptr + 41] = tmp_hack_rate;
-                } else {
-                    // 3 1 2
-                    tmp_hack_rate = gus_patch[gus_ptr + 41];
-                    gus_patch[gus_ptr + 41] = gus_patch[gus_ptr + 42];
-                    gus_patch[gus_ptr + 42] = tmp_hack_rate;
-                }
-            }
-
-#if 0
-            if ((env_time_table[gus_patch[gus_ptr + 40]] < env_time_table[gus_patch[gus_ptr + 41]]) && (env_time_table[gus_patch[gus_ptr + 41]] == env_time_table[gus_patch[gus_ptr + 42]])) {
-                uint8_t tmp_hack_rate = 0;
-                tmp_hack_rate = gus_patch[gus_ptr + 41];
-                gus_patch[gus_ptr + 41] = gus_patch[gus_ptr + 40];
-                gus_patch[gus_ptr + 42] = gus_patch[gus_ptr + 40];
-                gus_patch[gus_ptr + 40] = tmp_hack_rate;
-                tmp_hack_rate = gus_patch[gus_ptr + 47];
-                gus_patch[gus_ptr + 47] = gus_patch[gus_ptr + 46];
-                gus_patch[gus_ptr + 48] = gus_patch[gus_ptr + 46];
-                gus_patch[gus_ptr + 46] = tmp_hack_rate;
-            }
-#endif
-        }
+        // kill "echo" envelopes because users dont want/like them
+        // and this is what timidity does which is why we sounded different
+        // NOTE: This may cause some pats to sound different to what their authers intended
+        gus_patch[gus_ptr + 41] = 0x3f;
+        gus_patch[gus_ptr + 42] = 0x3f;
 
+        // lets set up the envelope data
         for (i = 0; i < 6; i++) {
             GUSPAT_INT_DEBUG("Envelope #",i);
             if (gus_sample->modes & SAMPLE_ENVELOPE) {
                 uint8_t env_rate = gus_patch[gus_ptr + 37 + i];
                 gus_sample->env_target[i] = 16448 * gus_patch[gus_ptr + 43 + i];
                 GUSPAT_INT_DEBUG("Envelope Level",gus_patch[gus_ptr+43+i]); GUSPAT_FLOAT_DEBUG("Envelope Time",env_time_table[env_rate]);
-                gus_sample->env_rate[i] = (int32_t) (4194303.0
+                gus_sample->env_rate[i] = (int32_t) (4194303.0f
                         / ((float) _WM_SampleRate * env_time_table[env_rate]));
                 GUSPAT_INT_DEBUG("Envelope Rate",gus_sample->env_rate[i]); GUSPAT_INT_DEBUG("GUSPAT Rate",env_rate);
                 if (gus_sample->env_rate[i] == 0) {
-                    _WM_ERROR_NEW("%s: Warning: found invalid envelope(%u) rate setting in %s. Using %f instead.",
+                    _WM_DEBUG_MSG("%s: Warning: found invalid envelope(%u) rate setting in %s. Using %f instead.",
                             __FUNCTION__, i, filename, env_time_table[63]);
-                    gus_sample->env_rate[i] = (int32_t) (4194303.0
+                    gus_sample->env_rate[i] = (int32_t) (4194303.0f
                             / ((float) _WM_SampleRate * env_time_table[63]));
                     GUSPAT_FLOAT_DEBUG("Envelope Time",env_time_table[63]);
                 }
             } else {
                 gus_sample->env_target[i] = 4194303;
-                gus_sample->env_rate[i] = (int32_t) (4194303.0
+                gus_sample->env_rate[i] = (int32_t) (4194303.0f
                         / ((float) _WM_SampleRate * env_time_table[63]));
                 GUSPAT_FLOAT_DEBUG("Envelope Time",env_time_table[63]);
             }
         }
 
         gus_sample->env_target[6] = 0;
-        gus_sample->env_rate[6] = (int32_t) (4194303.0
+        gus_sample->env_rate[6] = (int32_t) (4194303.0f
                 / ((float) _WM_SampleRate * env_time_table[63]));
 
         gus_ptr += 96;
@@ -952,16 +875,16 @@ struct _sample * _WM_load_gus_pat(const char *filename, int fix_release) {
          NOTE: This sets samples for full range decay
          */
         if (gus_sample->modes & SAMPLE_ENVELOPE) {
-            float samples_f = 0.0;
+            float samples_f = 0;
 
             if (gus_sample->modes & SAMPLE_CLAMPED) {
-                samples_f = (4194301.0 - (float)gus_sample->env_target[5]) / gus_sample->env_rate[5];
+                samples_f = (4194301.0f - (float)gus_sample->env_target[5]) / gus_sample->env_rate[5];
             } else {
                 if (gus_sample->modes & SAMPLE_SUSTAIN) {
-                    samples_f = (4194301.0 - (float)gus_sample->env_target[3]) / gus_sample->env_rate[3];
+                    samples_f = (4194301.0f - (float)gus_sample->env_target[3]) / gus_sample->env_rate[3];
                     samples_f += (float)(gus_sample->env_target[3] - gus_sample->env_target[4]) / gus_sample->env_rate[4];
                 } else {
-                    samples_f = (4194301.0 - (float)gus_sample->env_target[4]) / gus_sample->env_rate[4];
+                    samples_f = (4194301.0f - (float)gus_sample->env_target[4]) / gus_sample->env_rate[4];
                 }
                 samples_f += (float)(gus_sample->env_target[4] - gus_sample->env_target[5]) / gus_sample->env_rate[5];
             }
