@@ -58,7 +58,7 @@ typedef struct {
 } midi_descriptor;
 
 struct xmi_ctx {
-    uint8_t *src, *src_ptr;
+    const uint8_t *src, *src_ptr;
     uint32_t srcsize;
     uint32_t datastart;
     uint8_t *dst, *dst_ptr;
@@ -122,7 +122,7 @@ static void copy(struct xmi_ctx *ctx, char *b, uint32_t len)
 #define DST_CHUNK 8192
 static void resize_dst(struct xmi_ctx *ctx) {
     uint32_t pos = ctx->dst_ptr - ctx->dst;
-    ctx->dst = realloc(ctx->dst, ctx->dstsize + DST_CHUNK);
+    ctx->dst = (uint8_t *) realloc(ctx->dst, ctx->dstsize + DST_CHUNK);
     ctx->dstsize += DST_CHUNK;
     ctx->dstrem += DST_CHUNK;
     ctx->dst_ptr = ctx->dst + pos;
@@ -461,7 +461,7 @@ static const char mt32asgs[256] = {
     121, 0  /* 127 Jungle Tune set to Breath Noise */
 };
 
-int _WM_xmi2midi(uint8_t *in, uint32_t insize,
+int _WM_xmi2midi(const uint8_t *in, uint32_t insize,
                  uint8_t **out, uint32_t *outsize,
                  uint32_t convert_type) {
     struct xmi_ctx ctx;
@@ -479,16 +479,16 @@ int _WM_xmi2midi(uint8_t *in, uint32_t insize,
     ctx.convert_type = convert_type;
 
     if (ParseXMI(&ctx) < 0) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_NOT_XMI, NULL, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_XMI, NULL, 0);
         goto _end;
     }
 
     if (ExtractTracks(&ctx) < 0) {
-        _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_NOT_MIDI, NULL, 0);
+        _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_NOT_MIDI, NULL, 0);
         goto _end;
     }
 
-    ctx.dst = malloc(DST_CHUNK);
+    ctx.dst = (uint8_t *) malloc(DST_CHUNK);
     ctx.dst_ptr = ctx.dst;
     ctx.dstsize = DST_CHUNK;
     ctx.dstrem = DST_CHUNK;
@@ -543,13 +543,13 @@ static void DeleteEventList(midi_event *mlist) {
 /* Sets current to the new event and updates list */
 static void CreateNewEvent(struct xmi_ctx *ctx, int32_t time) {
     if (!ctx->list) {
-        ctx->list = ctx->current = calloc(1, sizeof(midi_event));
+        ctx->list = ctx->current = (midi_event *) calloc(1, sizeof(midi_event));
         ctx->current->time = (time < 0)? 0 : time;
         return;
     }
 
     if (time < 0) {
-        midi_event *event = calloc(1, sizeof(midi_event));
+        midi_event *event = (midi_event *) calloc(1, sizeof(midi_event));
         event->next = ctx->list;
         ctx->list = ctx->current = event;
         return;
@@ -560,7 +560,7 @@ static void CreateNewEvent(struct xmi_ctx *ctx, int32_t time) {
 
     while (ctx->current->next) {
         if (ctx->current->next->time > time) {
-            midi_event *event = calloc(1, sizeof(midi_event));
+            midi_event *event = (midi_event *) calloc(1, sizeof(midi_event));
             event->next = ctx->current->next;
             ctx->current->next = event;
             ctx->current = event;
@@ -571,7 +571,7 @@ static void CreateNewEvent(struct xmi_ctx *ctx, int32_t time) {
         ctx->current = ctx->current->next;
     }
 
-    ctx->current->next = calloc(1, sizeof(midi_event));
+    ctx->current->next = (midi_event *) calloc(1, sizeof(midi_event));
     ctx->current = ctx->current->next;
     ctx->current->time = time;
 }
@@ -751,7 +751,7 @@ static int32_t ConvertSystemMessage(struct xmi_ctx *ctx, const int32_t time,
     if (!ctx->current->len)
         return (i);
 
-    ctx->current->buffer = malloc(sizeof(uint8_t)*ctx->current->len);
+    ctx->current->buffer = (uint8_t *) malloc(sizeof(uint8_t)*ctx->current->len);
     copy(ctx, (char *) ctx->current->buffer, ctx->current->len);
 
     return (i + ctx->current->len);
@@ -910,7 +910,7 @@ static uint32_t ConvertListToMTrk(struct xmi_ctx *ctx, midi_event *mlist) {
 
         /* Never occur */
         default:
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(unrecognized event)", 0);
+            _WM_DEBUG_MSG("%s: unrecognized event", __FUNCTION__);
             break;
         }
     }
@@ -953,7 +953,7 @@ static uint32_t ExtractTracksFromXmi(struct xmi_ctx *ctx) {
 
         /* Convert it */
         if (!(ppqn = ConvertFiletoList(ctx))) {
-            _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, NULL, 0);
+            _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, NULL, 0);
             break;
         }
         ctx->timing[num] = ppqn;
@@ -980,7 +980,7 @@ static int ParseXMI(struct xmi_ctx *ctx) {
 
     file_size = getsrcsize(ctx);
     if (getsrcpos(ctx) + 8 > file_size) {
-badfile:    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
+badfile:    _WM_GLOBAL_ERROR(__FUNCTION__, __LINE__, WM_ERR_CORUPT, "(too short)", 0);
         return (-1);
     }
 
@@ -1001,7 +1001,7 @@ badfile:    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(
 
         /* XDIRless XMIDI, we can handle them here. */
         if (!memcmp(buf, "XMID", 4)) {
-            _WM_ERROR_NEW("Warning: XMIDI without XDIR");
+            _WM_DEBUG_MSG("Warning: XMIDI without XDIR");
             ctx->info.tracks = 1;
         }
         /* Not an XMIDI that we recognise */
@@ -1084,8 +1084,8 @@ badfile:    _WM_GLOBAL_ERROR(__FUNCTION__, __FILE__, __LINE__, WM_ERR_CORUPT, "(
 static int ExtractTracks(struct xmi_ctx *ctx) {
     uint32_t i;
 
-    ctx->events = calloc(ctx->info.tracks, sizeof(midi_event*));
-    ctx->timing = calloc(ctx->info.tracks, sizeof(int16_t));
+    ctx->events = (midi_event **) calloc(ctx->info.tracks, sizeof(midi_event*));
+    ctx->timing = (int16_t *) calloc(ctx->info.tracks, sizeof(int16_t));
     /* type-2 for multi-tracks, type-0 otherwise */
     ctx->info.type = (ctx->info.tracks > 1)? 2 : 0;
 
